package HotSaNICmod::OSdep;

# return module version
#
sub version {
  ($VERSION = '$Revision: 1.22 $') =~ s/.*(\d+\.\d+).*/$1/;
  return "$^O.pm $VERSION";
  }

# sample data
#
sub sample {
  my %args=@_;

  @sections = split " ", ($args{SECTIONS});
  foreach (@sections){
    if ($_ eq "cpu")    {read_cpu(%args)} 
    elsif ($_ eq "load")   {read_load(%args)}
    elsif ($_ eq "proc")   {read_proc(%args)}
    elsif ($_ eq "mem")    {read_mem(%args)}
    elsif ($_ eq "users")  {read_users(%args)}
    elsif ($_ eq "irq")    {read_interrupts(%args)}
    elsif ($_ eq "uptime") {read_uptime(%args)}
    }

  }

# users.rrd -> tty pty pts
# cpu.rrd -> cpuusr cpusys cpunic cpuidl
# load.rrd -> load15 load5 load1
# proc.rrd -> procslp procrun proczmb procstp
# mem.rrd (Linux) -> memfre memshr membuf memcac swpfre swpuse

#####################################################################################
# build CPU statistics
# in /proc/stat there is a CPU entry which provides us with the
# number of ticks the CPU was in all states ( user nice system idle )
# We simply have to gather this information and calculate the actual percentage :)
# To do this we have to remember the last read values of course!
#
sub read_cpu {
  my %args=@_;
  my $multicpu=0;

  # read actual state
  open(IN,"/proc/stat");
  while(<IN>) {
    chomp;
    if( index($_,"cpu") >=0 ) {
      ($cpu, $cpu1, $cpu2, $cpu3, $cpu4)=split;
      $cpu1{$cpu}=$cpu1;
      $cpu2{$cpu}=$cpu2;
      $cpu3{$cpu}=$cpu3;
      $cpu4{$cpu}=$cpu4;
      $cpu5{$cpu}=0;
      $cpu1old{$cpu}=0;
      $cpu2old{$cpu}=0;
      $cpu3old{$cpu}=0;
      $cpu4old{$cpu}=0;
      $cpu5old{$cpu}=0;
      if ($cpu eq "cpu1") { $multicpu=1; }
      }
    }
  close(IN);
  
  # read last state and store actual state
  # usr nice sys idle int
  my $DATFILE=$args{VARDIR}."/cpu.dat";
  if ( -e $DATFILE ) {
    open(CPU,$DATFILE);
    while (<CPU>) {
      $line=$_." 0 0 0 0 0 0";
      ($cpu, $cpu1old, $cpu2old, $cpu3old, $cpu4old, $cpu5old)=split / /,$line;
    
      if ($cpu !~ /cpu/) {
        $cpu5old=$cpu4old;
        $cpu4old=$cpu3old;
        $cpu3old=$cpu2old;
        $cpu2old=$cpu1old;
        $cpu1old=$cpu;
        $cpu="cpu";
        }
  
      $cpu1old{$cpu}=$cpu1old;
      $cpu2old{$cpu}=$cpu2old;
      $cpu3old{$cpu}=$cpu3old;
      $cpu4old{$cpu}=$cpu4old;
      $cpu5old{$cpu}=$cpu5old;
      }
    close(CPU);
    }
  
  open(CPU,">".$DATFILE);
  
  foreach $cpu (sort keys %cpu1) {
    print CPU $cpu." ".$cpu1{$cpu}." ".$cpu2{$cpu}." ".$cpu3{$cpu}." ".$cpu4{$cpu}." ".$cpu5{$cpu}."\n";
  
# calculate everything....
#

    $all{$cpu}=($cpu1{$cpu}+$cpu2{$cpu}+$cpu3{$cpu}+$cpu4{$cpu}+$cpu5{$cpu})-($cpu1old{$cpu}+$cpu2old{$cpu}+$cpu3old{$cpu}+$cpu4old{$cpu}+$cpu5old{$cpu});

    $cpuusr{$cpu}=($cpu1{$cpu}-$cpu1old{$cpu})/$all{$cpu};
    $cpunic{$cpu}=($cpu2{$cpu}-$cpu2old{$cpu})/$all{$cpu};
    $cpusys{$cpu}=($cpu3{$cpu}-$cpu3old{$cpu})/$all{$cpu};
    $cpuidl{$cpu}=($cpu4{$cpu}-$cpu4old{$cpu})/$all{$cpu};
    $cpuint{$cpu}=($cpu5{$cpu}-$cpu5old{$cpu})/$all{$cpu};

    HotSaNICmod::do_rrd($cpu,"U",time,$cpuusr{$cpu},$cpunic{$cpu},$cpusys{$cpu},$cpuidl{$cpu});
  
# on single-cpu machines only the global database has to be updated.
#
    last if ($multicpu==0);
  
    }
  close(CPU);
  }



#####################################################################################
# here we go with the load-average value ...
#
sub read_load {
  my %args=@_;
  open(IN,"/proc/loadavg"); ($load1,$load5,$load15)=split(/ /,<IN>); close(IN);
  HotSaNICmod::do_rrd("load","U",time,$load15,$load5,$load1);
  }



#####################################################################################
# now the processes. We have to check how many processes are sleeping,
# running, zombieng spwapped to disc or stopped. To do this we simply
# have to walk through the /proc tree, check all "stat" files of each
# process-id subdir and increment the according state-counter.
#
sub read_proc {
  my %args=@_;
  ($procslp,$procrun,$proczmb,$procstp,$procdsc)=(0,0,0,0,0);
  opendir (DIR,"/proc");@dir=readdir (DIR);closedir(DIR);
  foreach $entry (@dir) {
    # check if entry is a numeric one
    if (($entry =~ /^\d+$/) && ( -e "/proc/$entry/stat" )) {
      if (open (STAT,"/proc/$entry/stat")) {
        $_=<STAT>;
        close(STAT);
        if (index($_," S ") >=0 ) { $procslp++ }
        elsif (index($_," D ") >=0 ) { $procdsc++ }
        elsif (index($_," R ") >=0 ) { $procrun++ }
        elsif (index($_," Z ") >=0 ) { $proczmb++ }
        elsif (index($_," T ") >=0 ) { $procstp++ }
        }
      }
    }
  HotSaNICmod::do_rrd("proc","U",time,$procslp,$procrun,$proczmb,$procstp,$procdsc);
  }

#####################################################################################
# read users stats
#
sub read_users {
  my %args=@_;


  # build users-table first
  #
  my $command="/usr/bin/who";
  open FILE,"$command|" || HotSaNIClog::error("unable to run '$command': $!");
  my @users=(<FILE>);
  close FILE;

  ($tty,$pty,$pts)=(0,0,0);
  foreach (@users) {
    if (index($_," tty") >=0 ) { $tty++; }
    elsif (index($_," pty") >=0 ) { $pty++; }
    elsif (index($_," pts") >=0 ) { $pts++; }
    }
  HotSaNICmod::do_rrd("users","U",time,$tty,$pty,$pts);
  }

####################################################################################
# here we go with the memory and swapfile statistics...
#
sub read_mem {

  my %args=@_;
  my $KERNEL=HotSaNICparser::identify_kernel();

  open(IN,"/proc/meminfo");
  while(<IN>) {
    chomp;
    if (index($_,"Mem:")>=0) { (undef,$memtot,undef,$memfre,$memshr,$membuf,$memcac)=split; }
    elsif (index($_,"Swap:")>=0) { (undef,undef,$swpuse,$swpfre)=split; }

    if ($KERNEL>2.4) {
      # This is kernel >= 2.5 code
      if ($_ =~ /^MemTotal:/) { (undef,$memtot,undef)=split; $memtot*=1024; }
      elsif ($_ =~ /^MemFree:/) { (undef,$memfre,undef)=split; $memfre*=1024; }
      elsif ($_ =~ /^Buffers:/) { (undef,$membuf,undef)=split; $membuf*=1024; }
      elsif ($_ =~ /^Cached:/) { (undef,$memcac,undef)=split; $memcac*=1024; }
      elsif ($_ =~ /^SwapTotal:/) { (undef,$swptot,undef)=split; $swptot*=1024; }
      elsif ($_ =~ /^SwapFree:/) { (undef,$swpfre,undef)=split; $swpfre*=1024; }
      }

    }
  close(IN);

  if ($KERNEL>2.4) { $swpuse=$swptot-$swpfre; }
  $memshr=$memtot-$memfre-$membuf-$memcac;

  HotSaNICmod::do_rrd("mem","U",time,$memfre,$memshr,$membuf,$memcac,$swpfre,$swpuse);
  }

sub read_interrupts {
  my %args=@_;

  # intr 905974335 490612570 3015 0 0 0 3 3 3 2 71611549 0 145961329 197785844 0 0 17 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0

  my @stats;
  open FILE,"/proc/stat";
  while (<FILE>) { (undef,@stats)=split if /intr/; }
  close FILE;
    
  # need first 17 values from array
  HotSaNICmod::do_rrd("irq","U",time,@stats[0..16]);
  }



#####################################################################################
# here we go with the load-average value ...
#
sub read_uptime {
  my %args=@_;
  open(IN,"/proc/uptime"); ($up,$idle)=split(/ /,<IN>); close(IN);
  $up/=86400;
  $idle/=86400;

  HotSaNICmod::do_rrd("uptime","U",time,$up,$idle);
  }

1;
