package HotSaNICmod::OSdep;

# return module version
#
sub version {
  ($VERSION = '$Revision: 1.8 $') =~ s/.*(\d+\.\d+).*/$1/;
  return "$^O.pm $VERSION";
  }

# sample data
#
sub sample {
  my %args=@_;

  @sections = split " ", ($args{SECTIONS});
  foreach (@sections){
    if ($_ eq "cpu")	{read_cpu(%args)}
    if ($_ eq "load")   {read_load(%args)}
    if ($_ eq "proc")   {read_proc(%args)}
    if ($_ eq "mem")	{read_mem(%args)}
    if ($_ eq "users")  {read_users(%args)}
    }
  }

# users.rrd -> tty pty pts
# cpu.rrd -> cpuusr cpusys cpunic cpuidl
# load.rrd -> load15 load5 load1
# proc.rrd -> procslp procrun proczmb procstp
# mem.rrd (Linux) -> memfre memshr membuf memcac swpfre swpuse

#####################################################################################
# build CPU statistics
# in /proc/stat there is a CPU entry which provides us with the
# number of ticks the CPU was in all states ( user nice system idle )
# We simply have to gather this information and calculate the actual percentage :)
# To do this we have to remember the last read values of course!
#
sub read_cpu {
  my %args=@_;
  my $multicpu=0;

  # read actual state
  
  # assumed order is: user nice sys intr idle
   $cpuraw = `sysctl kern.cp_time`;
  chomp $cpuraw;
  ($cpu, $cpu1, $cpu2, $cpu3, $cpu4, $cpu5)=split / /, $cpuraw;
  if ($cpu eq "kern.cp_time:") { $cpu = "cpu"; }
  $cpu1{$cpu}=$cpu1;
  $cpu2{$cpu}=$cpu2;
  $cpu3{$cpu}=$cpu3;
  $cpu4{$cpu}=$cpu4;
  $cpu5{$cpu}=$cpu5;
  $cpu1old{$cpu}=0;
  $cpu2old{$cpu}=0;
  $cpu3old{$cpu}=0;
  $cpu4old{$cpu}=0;
  $cpu5old{$cpu}=0;
  if ($cpu eq "cpu1") { $multicpu=1; }

  # read last state and store actual state
  # usr nice sys idle int
  if ( -e "cpu.dat" ) {
    open(CPU,"cpu.dat");
    while (<CPU>) {
      $line=$_." 0 0 0 0 0 0";
      ($cpu, $cpu1old, $cpu2old, $cpu3old, $cpu4old, $cpu5old)=split / /,$line;
    
      if ($cpu !~ /cpu/) {
        $cpu5old=$cpu4old;
        $cpu4old=$cpu3old;
        $cpu3old=$cpu2old;
        $cpu2old=$cpu1old;
        $cpu1old=$cpu;
        $cpu="cpu";
        }
  
      $cpu1old{$cpu}=$cpu1old;
      $cpu2old{$cpu}=$cpu2old;
      $cpu3old{$cpu}=$cpu3old;
      $cpu4old{$cpu}=$cpu4old;
      $cpu5old{$cpu}=$cpu5old;
      }
    close(CPU);
    }
  
  open(CPU,">cpu.dat");
  
  foreach $cpu (sort keys %cpu1) {
    print CPU $cpu." ".$cpu1{$cpu}." ".$cpu2{$cpu}." ".$cpu3{$cpu}." ".$cpu4{$cpu}." ".$cpu5{$cpu}."\n";
  
# calculate everything....
#

    $all{$cpu}=($cpu1{$cpu}+$cpu2{$cpu}+$cpu3{$cpu}+$cpu4{$cpu}+$cpu5{$cpu})-($cpu1old{$cpu}+$cpu2old{$cpu}+$cpu3old{$cpu}+$cpu4old{$cpu}+$cpu5old{$cpu});

    $cpuusr{$cpu}=($cpu1{$cpu}-$cpu1old{$cpu})/$all{$cpu};
    $cpunic{$cpu}=($cpu2{$cpu}-$cpu2old{$cpu})/$all{$cpu};
    $cpusys{$cpu}=($cpu3{$cpu}-$cpu3old{$cpu})/$all{$cpu};
    $cpuidl{$cpu}=($cpu4{$cpu}-$cpu4old{$cpu})/$all{$cpu};
    $cpuint{$cpu}=($cpu5{$cpu}-$cpu5old{$cpu})/$all{$cpu};

    HotSaNICmod::do_rrd($cpu,"U",time,$cpuusr{$cpu},$cpunic{$cpu},$cpusys{$cpu},$cpuidl{$cpu},$cpuint{$cpu});
  
# on single-cpu machines only the global database has to be updated.
#
    last if ($multicpu==0);
  
    }
  close(CPU);
  }



#####################################################################################
# here we go with the load-average value ...
#
sub read_load {
  my %args=@_;
  (undef,undef,$load1,$load5,$load15)=split(/ /,`/usr/sbin/sysctl vm.loadavg`);
  HotSaNICmod::do_rrd("load","U",time,$load15,$load5,$load1);
  }



#####################################################################################
# now the processes. We have to check how many processes are sleeping,
# running, zombieng spwapped to disc or stopped. To do this we simply
# have to walk through the /proc tree, check all "stat" files of each
# process-id subdir and increment the according state-counter.
#
sub read_proc {
  my %args=@_;
  $procslp=$procrun=$proczmb=$procstp=$procdsc=0;
  @states=`/bin/ps axostate`;
  foreach (@states) {
    $procslp++ if (/^S/);
    $procdsc++ if (/^D/);
    $procrun++ if (/^R/);
    $proczmb++ if (/^Z/);
    $procstp++ if (/^T/);
    }

  HotSaNICmod::do_rrd("proc","U",time,$procslp,$procrun,$proczmb,$procstp,$procdsc);
  }

#####################################################################################
# read users stats
#
sub read_users {
  my %args=@_;

  @users=`who`;
  ($tty,$pty,$pts)=(0,0,0);
  foreach (@users) {
    if (index($_," tty") >=0 ) { $tty++; }
    elsif (index($_," pty") >=0 ) { $pty++; }
    elsif (index($_," pts") >=0 ) { $pts++; }
    }
  HotSaNICmod::do_rrd("users","U",time,$tty,$pty,$pts);
  }

####################################################################################
# here we go with the memory and swapfile statistics...
#
sub read_mem {
  my %args=@_;

  #  HotSaNICmod::do_rrd("mem","U",time,$memfree,$memwire,$memactv,$meminac,$memcach,$swpfre,$swpuse);
  }

1;
